@@ -8,6 +8,7 @@ import abc
 import asyncio
 import concurrent.futures
 import contextlib
+import gc
 import json
 import logging
 import os
@@ -31,12 +32,22 @@ class LockAcquisitionTimeoutError(TimeoutError):
 @dataclass
 class FileInfo:
     size: int      # in bytes
-    atime: float   # last access POSIX timestamp
     ctime: float   # creation POSIX timetamp
     mtime: float   # last modification POSIX timestamp
     details: Any   # platform-dependent
 
 
+def nogc(func, *args, **kwargs):
+    isgc = gc.isenabled()
+    if isgc:
+        gc.disable()
+    try:
+        return func(*args, **kwargs)
+    finally:
+        if isgc:
+            gc.enable()
+
+
 class Upath(abc.ABC):  # pylint: disable=too-many-public-methods
     _executor: concurrent.futures.ThreadPoolExecutor = None
 
@@ -131,7 +142,7 @@ class Upath(abc.ABC):  # pylint: disable=too-many-public-methods
             /e/f/g/xy.data
             /e/f/g/h/d/dd.txt
 
-        then with `overwrite=False`:
+        then with `exist_action='raise'`:
 
             self         target         outcome
             /a/b/c.txt   ../c           /a/b/c/c.txt
@@ -171,6 +182,15 @@ class Upath(abc.ABC):  # pylint: disable=too-many-public-methods
             if exist_action == 'skip':
                 logger.info(f"target {target!r} exists; skipped")
                 return 0
+            if exist_action == 'update':
+                sourceinfo = self.file_info()
+                targetinfo = target.file_info()
+                if (targetinfo.size == sourceinfo.size
+                        and targetinfo.mtime >= sourceinfo.mtime):
+                    # We're assuming that this suggests
+                    # the target file was copied from the source
+                    # previously.
+                    return 0
             logger.info("copying '%s' to '%s'", self, target)
             target.write_bytes(self.read_bytes(), overwrite=True)
             return 1
@@ -196,11 +216,13 @@ class Upath(abc.ABC):  # pylint: disable=too-many-public-methods
         a default value (e.g. 4) is used.
 
         `exist_action`: what to do when the target file already exists.
-        There are three possible values:
+        These are the possible values:
 
             'raise' (default): raise `FileExistsError`.
             'skip': skip this file; proceed to work on other files.
             'overwrite': overwrite the existing file.
+            'update': overwrite if source `mtime` is newer than target,
+                or source and target have diff size; otherwise skip.
 
         The behavior is analogous to the command `cp` in Linux:
 
@@ -222,7 +244,7 @@ class Upath(abc.ABC):  # pylint: disable=too-many-public-methods
         if exist_action is None:
             exist_action = 'raise'
         else:
-            assert exist_action in ('raise', 'skip', 'overwrite')
+            assert exist_action in ('raise', 'skip', 'overwrite', 'update')
 
         if target.isdir():
             target = target / self.name
@@ -263,7 +285,6 @@ class Upath(abc.ABC):  # pylint: disable=too-many-public-methods
 
         raise FileNotFoundError(self)
 
-    @ abc.abstractmethod
     def exists(self) -> bool:
         '''Return `True` if the path is an existing file or dir,
         `False` otherwise.
@@ -278,12 +299,12 @@ class Upath(abc.ABC):  # pylint: disable=too-many-public-methods
         '/a/b' exists, and is a dir;
         '/a/b/c' does not exist.
         '''
-        raise NotImplementedError
+        return self.isfile() or self.isdir()
 
     @ abc.abstractmethod
-    def file_info(self) -> FileInfo:
+    def file_info(self) -> Optional[FileInfo]:
         '''
-        If `self.isfile()` is `False`, raise `FileNotFoundError`.
+        If `self.isfile()` is `False`, return `None`.
         '''
         raise NotImplementedError
 
@@ -414,11 +435,17 @@ class Upath(abc.ABC):  # pylint: disable=too-many-public-methods
         '''
         raise NotImplementedError
 
-    def read_json(self, **kwargs):
-        return json.loads(self.read_text(**kwargs))
+    def read_json(self, *, no_gc: bool = True, **kwargs):
+        z = self.read_text(**kwargs)
+        if no_gc:
+            return nogc(json.loads, z)
+        return json.loads(z)
 
-    def read_pickle(self):
-        return pickle.loads(self.read_bytes())
+    def read_pickle(self, *, no_gc: bool = True):
+        z = self.read_bytes()
+        if no_gc:
+            return nogc(pickle.loads, z)
+        return pickle.loads(z)
 
     def read_text(self, *, encoding: str = 'utf-8', errors: str = 'strict'):
         # Refer to https://docs.python.org/3/library/functions.html#open
@@ -608,6 +635,15 @@ class Upath(abc.ABC):  # pylint: disable=too-many-public-methods
                 if exist_action == 'skip':
                     logger.info(f"target {target!r} exists; skipped")
                     return 0
+                if exist_action == 'update':
+                    sourceinfo = await self.a_file_info()
+                    targetinfo = await target.a_file_info()
+                    if (targetinfo.size == sourceinfo.size
+                            and targetinfo.mtime >= sourceinfo.mtime):
+                        # We're assuming that this suggests
+                        # the target file was copied from the source
+                        # previously.
+                        return 0
                 logger.info("copying '%s' to '%s'", self, target)
                 await target.a_write_bytes(
                     await self.a_read_bytes(), overwrite=True)
@@ -637,7 +673,7 @@ class Upath(abc.ABC):  # pylint: disable=too-many-public-methods
         if exist_action is None:
             exist_action = 'raise'
         else:
-            assert exist_action in ('raise', 'skip', 'overwrite')
+            assert exist_action in ('raise', 'skip', 'overwrite', 'update')
 
         if await target.a_isdir():
             target = target / self.name
@@ -658,7 +694,7 @@ class Upath(abc.ABC):  # pylint: disable=too-many-public-methods
             n = 0
             async for p in self.a_riterdir():
                 extra = str(p.path.relative_to(self.path))
-                k = await self._a_copy_file_to(
+                k = await p._a_copy_file_to(
                     target/extra,
                     exist_action=exist_action,
                 )
@@ -669,7 +705,7 @@ class Upath(abc.ABC):  # pylint: disable=too-many-public-methods
         tasks = []
         async for p in self.a_riterdir():
             extra = str(p.path.relative_to(self.path))
-            tasks.append(self._a_copy_file_to(
+            tasks.append(p._a_copy_file_to(
                 target/extra,
                 exist_action=exist_action,
                 semaphore=sem,
@@ -679,7 +715,7 @@ class Upath(abc.ABC):  # pylint: disable=too-many-public-methods
         return sum(nn)
 
     async def a_exists(self):
-        return await self._a_do(self.exists)
+        return (await self.a_isfile()) or (await self.a_isdir())
 
     async def a_file_info(self):
         return await self._a_do(self.file_info)
@@ -702,19 +738,27 @@ class Upath(abc.ABC):  # pylint: disable=too-many-public-methods
             yield
 
     async def a_ls(self):
-        return await self._a_do(self.ls)
+        return sorted((p async for p in self.a_iterdir()))
 
     async def a_read_bytes(self):
         return await self._a_do(self.read_bytes)
 
-    async def a_read_json(self, **kwargs):
-        return await self._a_do(self.read_json, **kwargs)
+    async def a_read_json(self, *, no_gc: bool = True, **kwargs):
+        z = await self.a_read_text(**kwargs)
+        if no_gc:
+            return nogc(json.loads, z)
+        return json.loads(z)
 
-    async def a_read_pickle(self):
-        return await self._a_do(self.read_pickle)
+    async def a_read_pickle(self, *, no_gc: bool = True):
+        z = await self.a_read_bytes()
+        if no_gc:
+            return nogc(pickle.loads, z)
+        return pickle.loads(z)
 
-    async def a_read_text(self, **kwargs):
-        return await self._a_do(self.read_text, **kwargs)
+    async def a_read_text(self, *,
+                          encoding: str = 'utf-8', errors: str = 'strict'):
+        return (await self.a_read_bytes()).decode(
+            encoding=encoding, errors=errors)
 
     async def a_rename(self, *args, **kwargs):
         return await self._a_do(self.rename, *args, **kwargs)
@@ -745,11 +789,25 @@ class Upath(abc.ABC):  # pylint: disable=too-many-public-methods
     async def a_write_bytes(self, *args, **kwargs):
         return await self._a_do(self.write_bytes, *args, **kwargs)
 
-    async def a_write_json(self, *args, **kwargs):
-        return await self._a_do(self.write_json, *args, **kwargs)
+    async def a_write_json(self, data, *, overwrite=False, **kwargs) -> int:
+        return await self.a_write_text(
+            json.dumps(data), overwrite=overwrite, **kwargs)
 
-    async def a_write_pickle(self, *args, **kwargs):
-        return await self._a_do(self.write_pickle, *args, **kwargs)
+    async def a_write_pickle(self, data, *, overwrite=False) -> int:
+        return await self.a_write_bytes(
+            pickle.dumps(data, protocol=pickle.HIGHEST_PROTOCOL),
+            overwrite=overwrite,
+        )
 
-    async def a_write_text(self, *args, **kwargs):
-        return await self._a_do(self.write_text, *args, **kwargs)
+    async def a_write_text(
+            self,
+            data: str,
+            *,
+            overwrite: bool = False,
+            encoding: str = 'utf-8',
+            errors: str = 'strict',
+    ) -> int:
+        n = len(data)
+        z = data.encode(encoding=encoding, errors=errors)
+        await self.a_write_bytes(z, overwrite=overwrite)
+        return n
